@@ -48,4 +48,8 @@ void playmode_changed_cb (GtkWidget *widget,
 			  GnomeCD *gcd);
 void open_preferences (GtkWidget *widget,
 		       GnomeCD *gcd);
+void open_track_editor (GtkWidget *widget,
+			GnomeCD *gcd);
+void volume_changed (GtkRange *range,
+		     GnomeCD *gcd);
 #endif
