@@ -47,9 +47,6 @@ typedef struct _CDDisplayTheme {
 } CDDisplayTheme;
 
 struct _CDDisplayPrivate {
-	GdkColor red;
-	GdkColor blue;
-
 	GnomeCDText *layout[CD_DISPLAY_END];
 	int max_width;
 	int height;
@@ -413,9 +410,14 @@ expose_event (GtkWidget *drawing_area,
 						     X_OFFSET, 
 						     Y_OFFSET + height,
 						     priv->layout[i]->layout,
-						     &priv->red, NULL);
+						     NULL, NULL);
+		}
+
+		if (i == CD_DISPLAY_LINE_INFO) {
+			height += 16;
+		} else {
+			height += priv->layout[i]->height;
 		}
-		height += priv->layout[i]->height;
 	}
 
 	return TRUE;
@@ -455,7 +457,7 @@ realize (GtkWidget *widget)
 	GdkColormap *cmap;
 
 	disp = CD_DISPLAY (widget);
-
+#if 0
 	cmap = gtk_widget_get_colormap (widget);
 	disp->priv->red.red = 65535;
 	disp->priv->red.green = 0;
@@ -468,7 +470,7 @@ realize (GtkWidget *widget)
 	disp->priv->blue.blue = 65535;
 	disp->priv->blue.pixel = 0;
 	gdk_color_alloc (cmap, &disp->priv->blue);
-
+#endif
 	gtk_widget_add_events (widget, GDK_BUTTON_PRESS_MASK |
 			       GDK_KEY_PRESS_MASK);
 
@@ -616,8 +618,8 @@ init (CDDisplay *disp)
 		
 		priv->layout[i] = g_new (GnomeCDText, 1);
 		
-		priv->layout[i]->text = g_strdup (default_text[i]);
-		priv->layout[i]->length = strlen (default_text[i]);
+		priv->layout[i]->text = g_strdup (_(default_text[i]));
+		priv->layout[i]->length = strlen (_(default_text[i]));
 		priv->layout[i]->layout = gtk_widget_create_pango_layout (GTK_WIDGET (disp), priv->layout[i]->text);
 		pango_layout_set_text (priv->layout[i]->layout,
 				       priv->layout[i]->text,
@@ -698,8 +700,13 @@ cd_display_set_line (CDDisplay *disp,
 	g_return_if_fail (new_str != NULL);
 
 	priv = disp->priv;
-	
+
 	text = priv->layout[line];
+	if (strcmp (new_str, text->text) == 0) {
+		/* Same, do nothing */
+		return;
+	}
+
 	height = priv->height - text->height;
 	
 	g_free (text->text);
