@@ -23,6 +23,20 @@
 
 #define DEFAULT_THEME "lcd"
 
+/* Debugging? */
+gboolean debug_mode = FALSE;
+
+void
+gcd_warning (const char *message,
+	     GError *error)
+{
+	if (debug_mode == FALSE) {
+		return;
+	}
+	
+	g_warning (message, error ? error->message : "(None)");
+}
+
 void
 gnome_cd_set_window_title (GnomeCD *gcd,
 			   const char *artist,
@@ -30,7 +44,7 @@ gnome_cd_set_window_title (GnomeCD *gcd,
 {
 	char *title;
 
-	if (artist == NULL &&
+	if (artist == NULL ||
 	    track == NULL) {
 		title = g_strdup ("Gnome-CD " VERSION);
 	} else {
@@ -76,7 +90,7 @@ skip_to_track (GtkWidget *item,
 	}
 	
 	if (gnome_cdrom_play (GNOME_CDROM (gcd->cdrom), track + 1, &msf, end_track, endmsf, &error) == FALSE) {
-		g_warning ("Error skipping %s", error->message);
+		gcd_warning ("Error skipping %s", error);
 		g_error_free (error);
 	}
 }
@@ -100,8 +114,17 @@ gnome_cd_build_track_list_menu (GnomeCD *gcd)
 		int i;
 		for (i = 0; i < gcd->disc_info->ntracks; i++) {
 			char *title;
+			CDDBSlaveClientTrackInfo *info;
 
-			title = g_strdup_printf ("%d - %s", i + 1, gcd->disc_info->track_info[i]->name);
+			info = gcd->disc_info->track_info[i];
+			if (info == NULL) {
+				title = g_strdup_printf ("%d - %s", i + 1,
+							 _("Unknown track"));
+			} else {
+				title = g_strdup_printf ("%d - %s", i + 1,
+							 info->name);
+			}
+			
 			item = gtk_menu_item_new_with_label (title);
 			g_free (title);
 			gtk_widget_show (item);
@@ -192,6 +215,23 @@ make_button_from_file (GnomeCD *gcd,
 	return make_button_from_widget (gcd, pixmap, func, tooltip, shortname);
 }
 
+static GtkWidget *
+make_button_from_pixbuf (GnomeCD *gcd,
+			 GdkPixbuf *pixbuf,
+			 GCallback func,
+			 const char *tooltip,
+			 const char *shortname)
+{
+	GtkWidget *pixmap;
+
+	g_return_val_if_fail (gcd != NULL, NULL);
+	g_return_val_if_fail (pixbuf != NULL, NULL);
+		
+	pixmap = gtk_image_new_from_pixbuf (pixbuf);
+
+	return make_button_from_widget (gcd, pixmap, func, tooltip, shortname);
+}
+
 static GtkWidget *
 make_button_from_stock (GnomeCD *gcd,
 			const char *stock,
@@ -240,10 +280,12 @@ window_destroy_cb (GtkWidget *window,
 	case GNOME_CD_PREFERENCES_STOP_OPEN:
 		gnome_cdrom_eject (gcd->cdrom, NULL);
 		break;
-
+		
+#ifdef HAVE_CDROMCLOSETRAY_IOCTL
 	case GNOME_CD_PREFERENCES_STOP_CLOSE:
 		gnome_cdrom_close_tray (gcd->cdrom, NULL);
 		break;
+#endif
 
 	default:
 		g_assert_not_reached ();
@@ -286,7 +328,7 @@ make_popup_menu (GnomeCD *gcd)
 	for (i = 0; menuitems[i].name != NULL; i++) {
 		GtkWidget *item, *image;
 
-		item = gtk_image_menu_item_new_with_mnemonic (menuitems[i].name);
+		item = gtk_image_menu_item_new_with_mnemonic (_(menuitems[i].name));
 		if (menuitems[i].icon != NULL) {
 			char *ext = strrchr (menuitems[i].icon, '.');
 
@@ -330,11 +372,14 @@ static GnomeCD *
 init_player (void) 
 {
 	GnomeCD *gcd;
+	GnomeCDRomStatus *status;
 	GtkWidget *display_box;
-	GtkWidget *top_hbox, *button_hbox, *side_vbox;
+	GtkWidget *top_hbox, *button_hbox, *option_hbox;
 	GtkWidget *button, *arrow;
 	GdkPixbuf *pixbuf;
 	GError *error = NULL;
+	GtkAdjustment *adj;
+	GtkWidget *slider;
 	char *fullname;
 
 	gcd = g_new0 (GnomeCD, 1);
@@ -373,7 +418,7 @@ init_player (void)
 	gcd->cdrom = gnome_cdrom_new (gcd->preferences->device, GNOME_CDROM_UPDATE_CONTINOUS, &error);
 	if (gcd->cdrom == NULL) {
 		if (error != NULL) {
-			g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
+			gcd_warning ("%s", error);
 			g_error_free (error);
 		}
 	} else {
@@ -386,9 +431,9 @@ init_player (void)
 		
 		dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_NONE,
-						 _("The CD device that is set is not a CDRom drive. This means that GnomeCD\n"
-						   "will be unable to run correctly. Press Set device to go to a dialog\n"
-						   "where you can set the device, or click Quit to quit GnomeCD"));
+						 _("%s\nThis means that GnomeCD"
+						   " will be unable to run correctly. Press Set device to go to a dialog "
+						   "where you can set the device, or click Quit to quit GnomeCD"), error->message);
 		gtk_dialog_add_buttons (GTK_DIALOG (dialog), GTK_STOCK_QUIT, GTK_RESPONSE_CLOSE,
 					_("Set device"), 1, NULL);
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog), 1);
@@ -427,22 +472,9 @@ init_player (void)
 
 	g_signal_connect (G_OBJECT (gcd->window), "destroy",
 			  G_CALLBACK (window_destroy_cb), gcd);
-	gcd->vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
+	gcd->vbox = gtk_vbox_new (FALSE, 1);
 	gcd->tooltips = gtk_tooltips_new ();
-	top_hbox = gtk_hbox_new (FALSE, 0);
-
-	/* Create app controls */
-	side_vbox = gtk_vbox_new (FALSE, 0);
-	button = make_button_from_stock (gcd, GTK_STOCK_INDEX, G_CALLBACK(open_track_editor), _("Open track editor"), _("Track editor"));
-	gtk_widget_set_sensitive (button, FALSE);
-	gtk_box_pack_start (GTK_BOX (side_vbox), button, FALSE, FALSE, 0);
-	gcd->trackeditor_b = button;
-
-	button = make_button_from_stock (gcd, GTK_STOCK_PREFERENCES, G_CALLBACK(open_preferences), _("Preferences"), _("Preferences"));
-	gtk_box_pack_start (GTK_BOX (side_vbox), button, FALSE, FALSE, 0);
-	gcd->properties_b = button;
-
-	gtk_box_pack_start (GTK_BOX (top_hbox), side_vbox, FALSE, FALSE, 0);
+	top_hbox = gtk_hbox_new (FALSE, 1);
 
 	/* Create the display */
 	display_box = gtk_vbox_new (FALSE, 1);
@@ -464,23 +496,62 @@ init_player (void)
 	
 	gtk_box_pack_start (GTK_BOX (display_box), gcd->display, TRUE, TRUE, 0);
 
+
+	gtk_box_pack_start (GTK_BOX (top_hbox), display_box, TRUE, TRUE, 0);
+
+	/* Volume slider */
+	gcd->slider = gtk_vscale_new_with_range (0.0, 255.0, 1.0);
+	gtk_range_set_inverted (GTK_RANGE (gcd->slider), TRUE);
+	gtk_scale_set_draw_value (GTK_SCALE (gcd->slider), FALSE);
+	gtk_box_pack_start (GTK_BOX (top_hbox), gcd->slider, FALSE, FALSE, 0);
+
+	gtk_box_pack_start (GTK_BOX (gcd->vbox), top_hbox, TRUE, TRUE, 0);
+
+	option_hbox = gtk_hbox_new (FALSE, 2);
+
+	/* Create app controls */
+	button = make_button_from_stock (gcd, GTK_STOCK_INDEX,
+					 G_CALLBACK(open_track_editor),
+					 _("Open track editor"),
+					 _("Track editor"));
+	gtk_widget_set_sensitive (button, FALSE);
+	gtk_box_pack_start (GTK_BOX (option_hbox), button, FALSE, FALSE, 0);
+	gcd->trackeditor_b = button;
+
+	button = make_button_from_stock (gcd, GTK_STOCK_PREFERENCES,
+					 G_CALLBACK(open_preferences),
+					 _("Open preferences"),
+					 _("Preferences"));
+	gtk_box_pack_start (GTK_BOX (option_hbox), button, FALSE, FALSE, 0);
+	gcd->properties_b = button;
+
 	gcd->tracks = gtk_option_menu_new ();
 	g_signal_connect (G_OBJECT (gcd->tracks), "changed",
 			  G_CALLBACK (skip_to_track), gcd);
 	gnome_cd_build_track_list_menu (gcd);
-	gtk_box_pack_start (GTK_BOX (display_box), gcd->tracks, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (option_hbox), gcd->tracks, FALSE, FALSE, 0);
 
-	gtk_box_pack_start (GTK_BOX (top_hbox), display_box, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (gcd->vbox), option_hbox, FALSE, FALSE, 0);
 	
-	gtk_box_pack_start (GTK_BOX (gcd->vbox), top_hbox, TRUE, TRUE, 0);
+	/* Get the initial volume */
+	if (gnome_cdrom_get_status (gcd->cdrom, &status, NULL) == TRUE) {
+		gtk_range_set_value (GTK_RANGE (gcd->slider),
+				     (double) status->volume);
+		g_free (status);
+	} else {
+		gcd_warning ("Error getting status: %s", NULL);
+	}
+
+	g_signal_connect (G_OBJECT (gcd->slider), "value-changed",
+			  G_CALLBACK (volume_changed), gcd);
 	
 	button_hbox = gtk_hbox_new (TRUE, 2);
 	
-  	button = make_button_from_file (gcd, "gnome-cd/media-prev.png", G_CALLBACK (back_cb), _("Previous track"), _("Previous"));
+  	button = make_button_from_pixbuf (gcd, gcd->theme->previous, G_CALLBACK (back_cb), _("Previous track"), _("Previous"));
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->back_b = button;
 
-	button = make_button_from_file (gcd, "gnome-cd/media-rew.png", NULL, _("Rewind"), _("Rewind"));
+	button = make_button_from_pixbuf (gcd, gcd->theme->rewind, NULL, _("Rewind"), _("Rewind"));
 	g_signal_connect (G_OBJECT (button), "button-press-event",
 			  G_CALLBACK (rewind_press_cb), gcd);
 	g_signal_connect (G_OBJECT (button), "button-release-event",
@@ -490,29 +561,23 @@ init_player (void)
 
 	/* Create the play and pause images, and ref them so they never
 	   get destroyed */
-	fullname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
-                   "gnome-cd/media-play.png", TRUE, NULL);
-	gcd->play_image = gtk_image_new_from_file (fullname);
+	gcd->play_image = gtk_image_new_from_pixbuf (gcd->theme->play);
 	g_object_ref (gcd->play_image);
-	g_free (fullname);
 
-	fullname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
-                   "gnome-cd/media-pause.png", TRUE, NULL);
-	gcd->pause_image = gtk_image_new_from_file (fullname);
+	gcd->pause_image = gtk_image_new_from_pixbuf (gcd->theme->pause);
 	gtk_widget_show (gcd->pause_image);
 	g_object_ref (gcd->pause_image);
-	g_free (fullname);
 
 	button = make_button_from_widget (gcd, gcd->play_image, G_CALLBACK (play_cb), _("Play / Pause"), _("Play"));
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->play_b = button;
 	gcd->current_image = gcd->play_image;
 	
-	button = make_button_from_file (gcd, "gnome-cd/media-stop.png", G_CALLBACK (stop_cb), _("Stop"), _("Stop"));
+	button = make_button_from_pixbuf (gcd, gcd->theme->stop, G_CALLBACK (stop_cb), _("Stop"), _("Stop"));
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->stop_b = button;
 
-	button = make_button_from_file (gcd, "gnome-cd/media-fwd.png", NULL, _("Fast forward"), _("Fast forward"));
+	button = make_button_from_pixbuf (gcd, gcd->theme->forward, NULL, _("Fast forward"), _("Fast forward"));
 	g_signal_connect (G_OBJECT (button), "button-press-event",
 			  G_CALLBACK (ffwd_press_cb), gcd);
 	g_signal_connect (G_OBJECT (button), "button-release-event",
@@ -520,24 +585,14 @@ init_player (void)
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->ffwd_b = button;
 
-	button = make_button_from_file (gcd, "gnome-cd/media-next.png", G_CALLBACK (next_cb), _("Next track"), _("Next track"));
+	button = make_button_from_pixbuf (gcd, gcd->theme->next, G_CALLBACK (next_cb), _("Next track"), _("Next track"));
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->next_b = button;
 	
-	button = make_button_from_file (gcd, "gnome-cd/a-eject.png", G_CALLBACK (eject_cb), _("Eject CD"), _("Eject"));
+	button = make_button_from_pixbuf (gcd, gcd->theme->eject, G_CALLBACK (eject_cb), _("Eject CD"), _("Eject"));
 	gtk_box_pack_start (GTK_BOX (button_hbox), button, TRUE, TRUE, 0);
 	gcd->eject_b = button;
 
-#if 0
-	button = make_button_from_file (gcd, "gnome-cd/mixer.png", G_CALLBACK (mixer_cb), _("Open mixer"), _("Open Mixer"));
-	gtk_box_pack_start (GTK_BOX (bottom_hbox), button, FALSE, FALSE, 0);
-	gcd->mixer_b = button;
-
-	arrow = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_NONE);
-	button = make_button_from_widget (gcd, arrow, NULL, _("Change volume"), _("Volume"));
-	gcd->volume_b = button;
-	gtk_box_pack_start (GTK_BOX (bottom_hbox), button, FALSE, FALSE, 0);
-#endif
 	gtk_box_pack_start (GTK_BOX (gcd->vbox), button_hbox, FALSE, FALSE, 0);
 
 	gtk_container_add (GTK_CONTAINER (gcd->window), gcd->vbox);
@@ -581,6 +636,10 @@ main (int argc,
 	GnomeCD *gcd;
 	GnomeClient *client;
 
+	if (g_getenv ("GNOME_CD_DEBUG")) {
+		debug_mode = TRUE;
+	}
+	
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
@@ -602,9 +661,12 @@ main (int argc,
 			  G_CALLBACK (client_die), gcd);
 
 	/* Do the start up stuff */
+#ifdef HAVE_CDROMCLOSETRAY_IOCTL
 	if (gcd->preferences->start_close) {
 		gnome_cdrom_close_tray (gcd->cdrom, NULL);
 	}
+#endif
+	
 	switch (gcd->preferences->start) {
 	case GNOME_CD_PREFERENCES_START_NOTHING:
 		break;
