@@ -14,6 +14,7 @@
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <sys/cdio.h>
+#include <sys/audioio.h>	
 #include <errno.h>
 
 #include "gnome-cd.h"
@@ -309,6 +310,7 @@ solaris_cdrom_eject (GnomeCDRom *cdrom,
 	}
 
 	g_free (status);
+	lcd->priv->ref_count = 0;
 	solaris_cdrom_close (lcd);
 	return TRUE;
 }
@@ -799,6 +801,8 @@ solaris_cdrom_get_status (GnomeCDRom *cdrom,
 	SolarisCDRomPrivate *priv;
 	GnomeCDRomStatus *realstatus;
 	struct cdrom_subchnl subchnl;
+	struct audio_info audioinfo;
+	int vol_fd;
 	int cd_status;
 
 	g_return_val_if_fail (status != NULL, TRUE);
@@ -808,6 +812,7 @@ solaris_cdrom_get_status (GnomeCDRom *cdrom,
 
 	*status = g_new (GnomeCDRomStatus, 1);
 	realstatus = *status;
+	realstatus->volume = 0;
 
 	if (solaris_cdrom_open (lcd, error) == FALSE) {
 		g_free (realstatus);
@@ -884,6 +889,17 @@ solaris_cdrom_get_status (GnomeCDRom *cdrom,
 		g_free (realstatus);
 		return FALSE;
 	}
+	/* get initial volume */
+	vol_fd = open ( "/dev/audioctl", O_RDWR);
+	if (ioctl (vol_fd, AUDIO_GETINFO, &audioinfo) < 0) {
+			g_warning ("(solaris_cdrom_get_status): AUDIO_GETINFO ioctl failed %s",
+				   strerror (errno));
+			realstatus->volume = -1;
+	} else {
+		realstatus->volume = audioinfo.play.gain;
+	}
+
+	close (vol_fd);
 
 	solaris_cdrom_close (lcd);
 
@@ -957,6 +973,118 @@ solaris_cdrom_close_tray (GnomeCDRom *cdrom,
 	return TRUE;
 }
 
+static gboolean
+solaris_cdrom_set_volume (GnomeCDRom *cdrom,
+			  int volume,
+                          GError **error)
+{
+	SolarisCDRom *lcd;
+	SolarisCDRomPrivate *priv;
+	struct audio_info audioinfo;
+	struct cdrom_volctrl vol;
+	int vol_fd;
+
+	lcd = SOLARIS_CDROM (cdrom);
+	priv = lcd->priv;
+	AUDIO_INITINFO (&audioinfo)
+
+	audioinfo.play.gain = volume;
+
+	vol_fd = open ("/dev/audioctl", O_RDWR);
+	if (vol_fd < 0) {
+		if (error) {
+			*error = g_error_new (GNOME_CDROM_ERROR,
+                                              GNOME_CDROM_ERROR_SYSTEM_ERROR,
+                                              "(solaris_cdrom_set_volume:1): ioctl failed %s",
+                                              strerror (errno));
+		}
+		close (vol_fd);
+		return FALSE;
+	}
+	
+	if (ioctl (vol_fd, AUDIO_SETINFO, &audioinfo) < 0) {
+		if (error) {
+			*error = g_error_new (GNOME_CDROM_ERROR,
+                                              GNOME_CDROM_ERROR_SYSTEM_ERROR,
+                                              "(solaris_cdrom_set_volume:1): ioctl failed %s",
+					      strerror (errno));
+		}
+
+		close (vol_fd);
+		return FALSE;
+	}
+
+	close (vol_fd);
+
+	if (solaris_cdrom_open (lcd, error) == FALSE) {
+		return FALSE;
+	}
+
+	vol.channel0 = volume;
+	vol.channel1 = vol.channel2 = vol.channel3 = volume;
+
+	if (ioctl (priv->cdrom_fd, CDROMVOLCTRL, &vol) < 0) {
+		if (error) {
+			*error = g_error_new (GNOME_CDROM_ERROR,
+					      GNOME_CDROM_ERROR_SYSTEM_ERROR,
+              				      "(solaris_cdrom_set_volume:1): ioctl failed %s",
+					      strerror (errno));
+                }
+
+		solaris_cdrom_close (lcd);
+		return FALSE;
+	}
+
+	solaris_cdrom_close (lcd);
+	return TRUE;
+}
+
+static gboolean
+solaris_cdrom_is_cdrom_device (GnomeCDRom *cdrom,
+			       const char *device,
+			       GError **error)
+{
+	int fd;
+	int vol_fd;
+	int volume;
+	struct audio_info audioinfo;
+	struct cdrom_volctrl vol;
+
+	vol_fd = open ( "/dev/audioctl", O_RDWR);
+	if (ioctl (vol_fd, AUDIO_GETINFO, &audioinfo) < 0) {
+			g_warning ("(solaris_cdrom_is_cdrom_device): AUDIO_GETINFO ioctl failed %s",
+                                   strerror (errno));
+	} else {
+                volume = audioinfo.play.gain;
+	}
+	close (vol_fd);
+
+	if (device == NULL || *device == 0) {
+		return FALSE;
+	}
+	
+	fd = open (device, O_RDONLY | O_NONBLOCK);
+	if (fd < 0) {
+		return FALSE;
+	}
+
+	vol.channel0 = volume;
+	vol.channel1 = vol.channel2 = vol.channel3 = volume;
+	/* Fire a harmless ioctl at the device. */
+	if (ioctl (fd, CDROMVOLCTRL, &vol) < 0) {
+		/* Failed, it's not a CDROM drive */
+		g_print ("%s is not a CDROM drive", device);
+		close (fd);
+		
+		return FALSE;
+	}
+	
+	g_print ("%s is a CDROM drive", device);
+	close (fd);
+
+	return TRUE;
+}
+
 static gboolean
 solaris_cdrom_get_cddb_data (GnomeCDRom *cdrom,
 			   GnomeCDRomCDDBData **data,
@@ -1032,6 +1160,19 @@ solaris_cdrom_set_device (GnomeCDRom *cdrom,
 		g_free (priv->cdrom_device);
 	}
 	priv->cdrom_device = g_strdup (device);
+	
+	if (gnome_cdrom_is_cdrom_device (GNOME_CDROM (lcd),
+					 priv->cdrom_device, error) == FALSE) {
+		if (error) {
+			*error = g_error_new (GNOME_CDROM_ERROR,
+					      GNOME_CDROM_ERROR_NOT_OPENED,
+					      "%s does not point to a valid CDRom device. This may be caused by:\n"
+					      "a) CD support is not compiled into Linux\n"
+					      "b) You do not have the correct permissions to access the CD drive\n"
+					      "c) %s is not the CD drive.\n",					     
+					      priv->cdrom_device, priv->cdrom_device);
+		}
+	}
 
 	/* Force the new CD to be scanned at next update cycle */
 	g_free (priv->recent_status);
@@ -1061,7 +1202,9 @@ class_init (SolarisCDRomClass *klass)
 	cdrom_class->back = solaris_cdrom_back;
 	cdrom_class->get_status = solaris_cdrom_get_status;
 	cdrom_class->close_tray = solaris_cdrom_close_tray;
-
+	cdrom_class->set_volume = solaris_cdrom_set_volume;
+	cdrom_class->is_cdrom_device = solaris_cdrom_is_cdrom_device;
+	
 	/* For CDDB */
   	cdrom_class->get_cddb_data = solaris_cdrom_get_cddb_data;
 
@@ -1171,8 +1314,20 @@ gnome_cdrom_new (const char *cdrom_device,
 	priv->cdrom_device = g_strdup (cdrom_device);
 	priv->update = update;
 
-	if (solaris_cdrom_open (cdrom, error) == FALSE) {
-		solaris_cdrom_close (cdrom);
+	solaris_cdrom_open (cdrom, error);
+	solaris_cdrom_close (cdrom);
+	if (gnome_cdrom_is_cdrom_device (GNOME_CDROM (cdrom),
+					 cdrom->priv->cdrom_device,
+					 error) == FALSE) {
+		if (error) {
+			*error = g_error_new (GNOME_CDROM_ERROR,
+					      GNOME_CDROM_ERROR_NOT_OPENED,
+					      "%s does not point to a valid CDRom device. This may be caused by:\n"
+					      "a) CD support is not compiled into Linux\n"
+					      "b) You do not have the correct permissions to access the CD drive\n"
+					      "c) %s is not the CD drive.\n",					     
+					      cdrom->priv->cdrom_device, cdrom->priv->cdrom_device);
+		}
 	}
 	
 	/* Force an update so that a status will always exist */
